/** From Java Tutorial, an example of
  * properly used exception 
  * The programs works as "cat" utility command on UNIX/Linux
  */

import java.io.*;

public class Cat {

	public static void cat(String fileName) {
		RandomAccessFile input = null;
    		String line = null;

    		try {
        		input = new RandomAccessFile(fileName, "r"); //may throw
        		while ((line = input.readLine()) != null) { //may throw
            		System.out.println(line);
        		}
        		return;
			// _checked_ FNFE is a part of RandomAccessFile constructor's signature
			// and it must be caught (to compile)  
			// FNFE or its supeclass can be caught
			// but FNFE goes first or it'll be missed
    		} catch (FileNotFoundException fnf) {
				System.err.println(fnf.getMessage());
			}
    		/*IOE is the parent to FNFE
    		 it's included to deal with E 
    		 that readLine may throw
			*/
			catch (IOException ioe) { 
    			System.err.println(ioe.getMessage()); 

    		} finally {
        		if (input != null) {
        			try {
            			input.close(); // the close() method throws a checked E
        			} catch (IOException e) {
        				// do nothing apart from catching
    				}
        		}
    		}
	}

	public static void main(String[] args) {
		cat(args[0]);
	}
}
